/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_E_2_6_a_i.cpp

     Summary:           write several elements before the left bound of the first dimension

     Test description:  the pointer to the multidimensional array is a default member variable; the allocation is in the constructor; the write is in a member function; "int" is used for the datatype of the array

     Error line:        98

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, allocated memory, default member variable, destructor, left bound, multidimensional array, out-of-bounds, write

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef NCOLUMNS
#define NCOLUMNS 100
#endif

#ifndef NROWS
#define NROWS 123
#endif

#ifndef GAP
#define GAP 50
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the multi dimensional array
        int **arrayA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // Allocate "main" array
    cout << "allocate memory for arrayA" << endl;
    arrayA=new int*[NROWS];

    // Allocate each member of the "main" array
    for (int i=0; i < NROWS; ++i) {
        arrayA[i]=new int[NCOLUMNS];
    }
}

TestClass :: ~TestClass() {
    // Delete each member of the "main" array
    cout << "deallocate the memory of arrayA" << endl;
    for (int i=0; i < NROWS; ++i) {
        delete[] arrayA[i];
    }

    // Delete "main" array
    delete[] arrayA;
}

void TestClass :: runA() {
    // initialize the array
    cout << "fill array with random values" << endl;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            arrayA[i][j]=rand();
        }
    }

    // outsmart dead code elimination
    int i=-GAP;
    int j=0;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=NROWS * rand();
        j=NCOLUMNS * rand();
    }

    // write several elements before the left bound
    cout << "write array[" << i << "][" << j << "]" << endl;
    int value=rand();
    arrayA[i][j]=value;

    // print out ptrA to prevent dead code elimination
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            value=value && arrayA[i][j];
        }
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
