/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_H_2_1_a_ofs.cpp

     Summary:           write to a file that was never opened

     Test description:  the file stream is a default member variable; the write is in a member function; "ofstream" is used for the file stream

     Error line:        67

     Support files:     Not needed

     Env. requirements: Write access to create a file for testing

     Keywords:          write, default member variable, member function, unopened file stream

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>
#include <fstream.h>

#ifndef FILENAME
#define FILENAME "fileA.txt"
#endif

#ifndef TESTSTRING
#define TESTSTRING "hello world"
#endif

#ifndef EXITCODE_ERROR
#define EXITCODE_ERROR 2
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // declaration of the file stream
        ofstream streamA;

    public:
        void runA();
};

void TestClass :: runA() {
    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        streamA.open(FILENAME, ios::out);
        if (!streamA) {
            cerr << "Cannot open file " << FILENAME << "." << endl;
            exit(EXITCODE_ERROR);
        }
    }

    // try to write to a unopened stream
    cout << "write " << TESTSTRING << " to the file " << FILENAME << endl;
    streamA << TESTSTRING;

    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        streamA.close();
    }
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
