/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_I_1_1_c_f.cpp

     Summary:           use "new" twice with the same pointer

     Test description:  the pointer is a default member variable; one allocation is in the constructor of the inheriting class; the second allocation is in a virtual member function; one deallocation is in the destructor of the base inheriting; "float" is used for the base datatype of the pointer

     Error line:        69

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, virtual member function, allocate twice, default member variable, destructor, new, same pointer

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClassBase {
    protected:
        virtual void runA()=0;
};

class TestClass : public TestClassBase {
        // this is the declaration of the pointer
        float *ptrA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // memory allocation
    cout << "allocation by using new" << endl;
    ptrA=new float;

    // initialize
    *ptrA=rand();

    // outsmart dead code elimination
    cout << "ptrA " << &ptrA << ", " << ptrA << ", " << *ptrA << endl;
}

TestClass :: ~TestClass() {
    // memory deallocation
    cout << "deallocation by using delete" << endl;
    delete ptrA;
}

void TestClass :: runA() {
    // memory allocation
    cout << "allocation by using new" << endl;
    ptrA=new float;

    // initialize
    *ptrA=rand();

    // outsmart dead code elimination
    cout << "ptrA " << &ptrA << ", " << ptrA << ", " << *ptrA << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
