/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_F_2_1_a_i.cpp

     Summary:           refer to one element before the left bound of a dynamically allocated array

     Test description:  the pointer to the array is a default member variable; the allocation is in the constructor; the declaration of the reference is in a member function; "int" is used for the datatype of the array

     Error line:        75

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, allocated memory, array, default member variable, destructor, left bound, out-of-bounds, read, reference

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef SIZE
#define SIZE 100
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the array
        int *arrayA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // allocate the array
    cout << "allocate memory for arrayA" << endl;
    arrayA=new int [SIZE];
}

TestClass :: ~TestClass() {
    // delete the array
    delete [] arrayA;
}

void TestClass :: runA() {
    // fill the array with random values
    for ( int i=0; i < SIZE; i++) {
        arrayA[i]=rand();
    }

    // outsmart dead code elimination
    int i=-1;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=SIZE * rand();
    }

    // one element before the left bound
    cout << "array[" << i << "] with bounds 0:" << SIZE-1 << endl;

    // declare a reference to an element of the array
    int & refA=arrayA[i];

    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        int value=refA;
        for ( int i=0; i < SIZE; i++) {
            value=value && arrayA[i];
        }
        cout << value << endl;
    }
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
