/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_F_3_2_a_d.cpp

     Summary:           write one element before the left bound of a dynamically allocated array

     Test description:  the pointer to the array is a default member variable; the allocation is in the constructor; the write is in a member function; "double" is used for the datatype of the array

     Error line:        78

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, allocated memory, array, default member variable, destructor, left bound, out-of-bounds, pointer, write

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef SIZE
#define SIZE 100
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the array
        double *arrayA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // allocate the array
    cout << "allocate memory for arrayA" << endl;
    arrayA=new double [SIZE];
}

TestClass :: ~TestClass() {
    // delete the array
    delete [] arrayA;
}

void TestClass :: runA() {
    // declare a pointer and let it point to the array
    double *ptrA=&arrayA[0];

    // initialize the array with random values
    cout << "fill the array with random values" << endl;
    for ( int i=0; i < SIZE; i++) {
        arrayA[i]=rand();
    }

    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        ptrA=&arrayA[SIZE * rand()];
    }

    // write one element before the left bound
    ptrA--;
    cout << "write ptrA=" << ptrA << " arrayA=" << arrayA << endl;
    double value=rand();
    *ptrA=value;

    // print out ptrA to prevent dead code elimination
    for ( int i=0; i < SIZE; i++) {
        value=value && arrayA[i];
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
