/*
* Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle,
* James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, 
* and Ziyu Zhang, All rights reserved.
* Licensed under the Educational Community License version 1.0.
* See the full agreement at http://rted.public.iastate.edu/ .
*/
/*
*    Test name:		c_E_5_2_a.c
*
*    Test description:	Write one element before the lower bound of  
*			a dynamically allocated multidimensional array 
*			using a subscript.
*			Declare and use the array in a block.
*
*    Error line:        74
*
*    Support files:     index_array.txt, TEST_CONST.H, TEST_PARAM.H
*    
*    Env. requirements: Not needed
*
*    Keywords:		out-of-bounds
*			array indexing
*			dynamically allocated array
*			multidimensional array
*			main function
*			block
*			fence-post error
*			off-by-one error
*			write
*                       int
*
*    Last modified:	July 30, 2005
*
*    Programmer:        Olga Taborskaia, Iowa State University
*/

#include "TEST_PARAM.H"

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

#ifndef EXITCODE_ERROR
#define EXITCODE_ERROR 2
#endif

int main(void)
{
   int a = 0, sum = 0;
   int i, j;

   {
     int **ptr;
     ptr = (int **)malloc(N*sizeof(int *));
     if(!ptr)
      { 
        printf("Out of memory.\n");
        exit(EXITCODE_ERROR);
      }   
     for(i=0; i<N; i++)
      {
         ptr[i] = (int *)malloc(N*sizeof(int));
         if(!ptr[i])
          { 
            printf("Out of memory.\n");
            exit(EXITCODE_ERROR);
          }   
      }
     for(i=0; i<N; i++)
         for(j=0; j<N; j++)
             ptr[i][j] = i+j;
     j = j - N - 1;
     ptr[0][j] = a;  /* write one element before the lower bound */

     for(i=0; i<N; i++)           /* use elements of the array */
         sum = sum + ptr[1][i];   /* to prevent code elimination */ 

     for(i=0; i<N; i++)
         free(ptr[i]);
     free(ptr);
   }
   
   ret();
   if(ret2==0) {                  /* the value of ret2 = 1 */
     printf("Variable sum=%d", sum);
   } 
   
   return EXITCODE_OK; 
}
